package com.bjlt.spider.chains.abstractchains;


import com.bjlt.spider.entity.PlatformMenuVo;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.search.service.PlatformMenuService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespEntity;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.sql.SQLException;
import java.util.List;

import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.preFixToBean;
import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.setBeanIntoRedis;

/**
 * @author Scott
 * @Description:
 * @Date: Create in 09:37 2017/10/13
 */
@Slf4j
@Component
public abstract class AbstractHandler {
    /**
     * 持有下一个处理请求的对象 也就是下一个控制点
     */
    @Getter
    @Setter
    protected AbstractHandler nextHandler;

    @Autowired
    private PlatformMenuService menuService;

    @Autowired
    protected RedisTemplate<String, String> redisTemplate;

    /**
     * 根据 request 获取 platforId
     */

    public PlatformMenuVo getPlatformId(String cid) {
        Object menuObj = preFixToBean(RedisPrefix.PLATFORMMENU.getPrefix() + cid, redisTemplate, PlatformMenuVo.class);

        //第一次处理的时候 先从缓存中取对象  如果有的话 直接从缓存拿

        PlatformMenuVo platformMenuInDb;
        if (menuObj != null) {
            platformMenuInDb = (PlatformMenuVo) menuObj;
        } else {
            //根据 CID 取出此平台是否上线的各种状态 数据库里面只能有一条
            List<PlatformMenuVo> platformMenuVos = null;
            try {
                platformMenuVos = menuService.listPlatformMenuVo(PlatformMenuVo.builder().cid(cid).build());
                if (platformMenuVos == null || platformMenuVos.size() != 1) {
                    log.error("服务平台配置数据库错误 {} ", cid);
                    return null;
                }
            } catch (SQLException e) {
                log.error("服务平台配置数据库错误 {} ", cid);
                return null;
            }

            platformMenuInDb = platformMenuVos.get(0);
            //然后刷新缓存
            setBeanIntoRedis(platformMenuInDb, RedisPrefix.PLATFORMMENU.getPrefix() + cid, redisTemplate, RedisPrefix.PLATFORMMENU);
        }
        return platformMenuInDb;
    }


    /**
     * 抽象方法 处理请求
     *
     * @param searchRequestAfterOta
     * @return
     */
    public abstract RespEntity handleRequest(@RequestBody SearchRequestAfterOta searchRequestAfterOta) throws SQLException;

}
