package com.bjlt.spider.chains.searchchaines;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.chains.abstractchains.AbstractHandler;
import com.bjlt.spider.entity.AirLineExchangeVo;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.search.service.AirLineExchangeService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespEntity;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.preFixToSet;
import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.setBeanIntoRedisWithSet;

/**
 * @author Scott
 * @author: Scott
 * @Description: 城市代码转换为机场代码 重新 set 到 lines  里面
 * Map<String,Map<String,String>>  <数据源 ID，<出发机场代码，到达机场代码>>
 * @Date: Create in 11:44 2017/10/14
 */
@Component
@Slf4j
public class AirCodeExchangeHandler extends AbstractHandler {


    @Autowired
    private AirLineExchangeService exchangeService;

    @Override
    public RespEntity handleRequest(SearchRequestAfterOta searchRequestAfterOta) throws SQLException {
        log.info("---------开始城市代码转换---------");

        String fromCity = searchRequestAfterOta.getFromCity();
        String toCity = searchRequestAfterOta.getToCity();

        //此时不能区分数据源
        Object airLineExchangeFrom = preFixToSet(RedisPrefix.AIRLINEEXCHANGE.getPrefix() + fromCity, redisTemplate, AirLineExchangeVo.class);

        Object airLineExchangeTo = preFixToSet(RedisPrefix.AIRLINEEXCHANGE.getPrefix() + toCity, redisTemplate, AirLineExchangeVo.class);

        Set<AirLineExchangeVo> setFrom = Sets.newHashSet();
        // 把 出发城市转为 出发机场和对应的数据源
        if (airLineExchangeFrom != null) {
            setFrom = (Set<AirLineExchangeVo>) airLineExchangeFrom;
        } else {
            //查询数据库  目前还不能确定航司
            List<AirLineExchangeVo> airLineExchangeVos = exchangeService.listAirLineExchangeVo(AirLineExchangeVo.builder().cityCode(fromCity).build());
            if (airLineExchangeVos != null && airLineExchangeVos.size() != 0) {
                Set<AirLineExchangeVo> finalSet = setFrom;
                airLineExchangeVos.forEach(t -> finalSet.add(t));
                setFrom = finalSet;

                setFrom.forEach(t -> setBeanIntoRedisWithSet(t, RedisPrefix.AIRLINEEXCHANGE.getPrefix() + fromCity, redisTemplate));
            }
        }
        //把抵达城市 转换为对应的机场代码
        Set<AirLineExchangeVo> setTo = Sets.newHashSet();
        if (airLineExchangeTo != null) {
            setTo = (Set<AirLineExchangeVo>) airLineExchangeTo;
        } else {
            //查询数据库  目前还不能确定航司
            List<AirLineExchangeVo> airLineExchangeVos = exchangeService.listAirLineExchangeVo(AirLineExchangeVo.builder().cityCode(toCity).build());
            if (airLineExchangeVos != null && airLineExchangeVos.size() != 0) {
                Set<AirLineExchangeVo> finalSet = setTo;
                airLineExchangeVos.forEach(t -> finalSet.add(t));
                setTo = finalSet;
                setTo.forEach(t -> setBeanIntoRedisWithSet(t, RedisPrefix.AIRLINEEXCHANGE.getPrefix() + toCity, redisTemplate));

            }
        }
        searchRequestAfterOta.setFromCityCodes(setFrom);

        searchRequestAfterOta.setToCityCodes(setTo);

        List<Map<String,String>> lines = exchangeLines(searchRequestAfterOta);

        searchRequestAfterOta.setLines(lines);
        log.info("---------城市代码转换完毕---------");
        if (getNextHandler() != null) {
            return nextHandler.handleRequest(searchRequestAfterOta);
        }

        return RespEntity.success(null);
    }


    public List<Map<String,String>> exchangeLines(SearchRequestAfterOta searchRequestAfterOta) {
        List<Map<String,String>> lines = Lists.newArrayList();
        List<String> fromLine = Lists.newArrayList();
        List<String> toLine = Lists.newArrayList();
        //数据源 和 出发机场代码 有很多个不重复的
        Set<AirLineExchangeVo> fromCityCodes = searchRequestAfterOta.getFromCityCodes();

        Set<AirLineExchangeVo> toCityCodes = searchRequestAfterOta.getToCityCodes();

        //第一种情况 出发城市和到达城市 都需要转换
        if (fromCityCodes.size() != 0 && toCityCodes.size() != 0) {
            Iterator<AirLineExchangeVo> iteratorFrom = fromCityCodes.iterator();

            while (iteratorFrom.hasNext()) {
                Object o = iteratorFrom.next();
                AirLineExchangeVo nextFrom;
                if (o instanceof String) {
                    nextFrom = JSONObject.parseObject(o.toString(), AirLineExchangeVo.class);
                } else {
                    nextFrom = (AirLineExchangeVo) o;
                }
                String airplaineCodeFrom = nextFrom.getAirplaineCode();
                fromLine.add(airplaineCodeFrom);
            }
            Iterator<AirLineExchangeVo> iteratorTo = toCityCodes.iterator();
            while (iteratorTo.hasNext()) {
                Object t = iteratorTo.next();
                AirLineExchangeVo nextTo;
                if (t instanceof String) {
                    nextTo = JSONObject.parseObject(t.toString(), AirLineExchangeVo.class);
                } else {
                    nextTo = (AirLineExchangeVo) t;
                }
                String airplaineCodeTo = nextTo.getAirplaineCode();
                toLine.add(airplaineCodeTo);
            }
            fromLine.forEach(from->
                toLine.forEach(to->
                    lines.add(ImmutableMap.of(from,to))
                )
            );
        }


        // 第二种 情况 出发城市需要转换 到达城市不需要转换
        if (fromCityCodes.size() != 0 && toCityCodes.size() == 0) {
            fromCityCodes.stream().map(t ->
                    lines.add(ImmutableMap.of(t.getAirplaineCode(), searchRequestAfterOta.getToCity()))
            );
        }
        //第三种情况

        if (fromCityCodes.size() == 0 && toCityCodes.size() != 0) {
            toCityCodes.stream().map(t ->
                    lines.add(ImmutableMap.of(searchRequestAfterOta.getFromCity(), t.getAirplaineCode())));
        }

        //第四种情况 都不需要转换的时候
        lines.add(ImmutableMap.of(searchRequestAfterOta.getFromCity(),searchRequestAfterOta.getToCity()));
        return lines;
    }
}
