package com.bjlt.spider.chains.searchchaines;

import com.bjlt.spider.chains.abstractchains.AbstractHandler;
import com.bjlt.spider.entity.AirSourceLineVo;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.entity.UnKnownLineVo;
import com.bjlt.spider.search.service.AirSourceLineService;
import com.bjlt.spider.search.service.UnKnownLineService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespCode;
import com.bjlt.spider.util.RespEntity;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.preFixToSet;
import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.setBeanIntoRedisWithSet;

/**
 * @author: Scott
 * @Description: 根据航线 把对应 的航司找出来 如果没有需要记录
 * @Date: Create in 11:53 2017/10/14
 */
@Slf4j
@Component
public class AirLineConfigHandler extends AbstractHandler {


    @Autowired
    private AirSourceLineService sourceLineService;

    @Autowired
    private UnKnownLineService unKnownLineService;


    @Override
    public RespEntity handleRequest(SearchRequestAfterOta vo) throws SQLException {
        log.info("---------航线匹配开始---------");

        List<Map<String, String>> lines = vo.getLines();

        List<AirSourceLineVo> airSourceLineVos = Lists.newArrayList();


        lines.forEach(t -> t.forEach((k, v) -> {
            try {
                //先在缓存里面查一遍
                Object o = preFixToSet(RedisPrefix.AIRSOURCELINE.getPrefix() + k + "->" + v, redisTemplate, AirSourceLineVo.class);

                if (o == null) {
                    List<AirSourceLineVo> vos = sourceLineService.listAirSourceLineVo(AirSourceLineVo.builder().fromCity(k).toCity(v).build());

                    vos.forEach(s -> setBeanIntoRedisWithSet(s, RedisPrefix.AIRSOURCELINE.getPrefix() + k + "->" + v, redisTemplate));
                    airSourceLineVos.addAll(vos);
                } else {

                    Set<AirSourceLineVo> vos = (Set<AirSourceLineVo>) o;
                    airSourceLineVos.addAll(vos);
                }


            } catch (SQLException e) {
                log.error("AirSourceLineVo search error", e);
            }
        }));


        if (airSourceLineVos.size() == 0) {
            //将找不到航线的航班记录下来

            //TODO 可以优化
            lines.forEach(t -> t.forEach((k, v) ->
                    unKnownLineService.insertSelective(UnKnownLineVo.builder().fromCity(k).toCity(v).build())));

            return RespEntity.error(RespCode.AIR_CANTFIND_ERROR);
        } else {
            vo.setAirSourceLines(airSourceLineVos);
        }

        log.info("---------航线匹配结束---------");
        if (getNextHandler() != null) {
            return nextHandler.handleRequest(vo);
        }
        return RespEntity.success(null);

    }
}
