package com.bjlt.spider.chains.searchchaines;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.chains.abstractchains.AbstractHandler;
import com.bjlt.spider.entity.AirSourceLineVo;
import com.bjlt.spider.entity.PlatformSourceVo;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.search.service.PlatformSourceService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.utils.RedisKeyAndBeanUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author: Scott
 * @Description: 平台数据源 有可能某些平台不要某些航司
 * @Date: Create in 17:57 2017/10/16
 */
@Component
public class AirSourceControHandler extends AbstractHandler {

    @Autowired
    private PlatformSourceService platformSourceService;


    @Override
    public RespEntity handleRequest(SearchRequestAfterOta vo) throws SQLException {
        //根据数据源把他所支持航司取出来
        String cid = vo.getCid();
        Integer platformId = getPlatformId(cid).getId();

        Set<String> set = RedisKeyAndBeanUtil.preFixToSet(RedisPrefix.AIRSOURCEPLATFORM.getPrefix() + platformId, redisTemplate, AirSourceLineVo.class);
        List<AirSourceLineVo> airSourceLines = vo.getAirSourceLines();

        Iterator<AirSourceLineVo> iterator = airSourceLines.iterator();

        List<AirSourceLineVo> newAirSourceLines = Lists.newArrayList();
        while (iterator.hasNext()){
            Object next = iterator.next();
            if (next instanceof String){
                AirSourceLineVo airSourceLineVo = JSONObject.parseObject(next.toString()).toJavaObject(AirSourceLineVo.class);
                newAirSourceLines.add(airSourceLineVo);
            }else {
                newAirSourceLines.add((AirSourceLineVo) next);
            }
        }

        airSourceLines = newAirSourceLines;
        List<PlatformSourceVo> vos = Lists.newArrayList();
        if (set == null) {
            vos = platformSourceService.listPlatformSourceVo(PlatformSourceVo.builder().cid(cid).status(1).build());
            vos.forEach(t -> RedisKeyAndBeanUtil.setBeanIntoRedisWithSet(t, RedisPrefix.AIRSOURCEPLATFORM.getPrefix() + platformId, redisTemplate));
        } else {
            List<PlatformSourceVo> finalVos = vos;
            set.forEach(t->
                    finalVos.add(JSONObject.parseObject(t).toJavaObject(PlatformSourceVo.class))
            );
            vos = finalVos;
        }


        List<String> collect = vos.stream().map(t -> t.getSourceGroupId()).collect(Collectors.toList());

        List<String> sources = airSourceLines.stream().map(t -> t.getAirSourceID()).filter(s -> collect.contains(s)).collect(Collectors.toList());

        airSourceLines = airSourceLines.stream().filter(t -> sources.contains(t.getAirSourceID())).collect(Collectors.toList());

        vo.setAirSourceLines(airSourceLines);

        if (getNextHandler() != null) {
            return nextHandler.handleRequest(vo);
        }
        return RespEntity.success(null);

    }
}
