package com.bjlt.spider.chains.searchchaines;


import com.bjlt.spider.chains.abstractchains.AbstractHandler;
import com.bjlt.spider.entity.PlatformMenuVo;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.search.service.PlatformMenuService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespCode;
import com.bjlt.spider.util.RespEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.SQLException;
import java.util.List;

import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.preFixToBean;
import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.setBeanIntoRedis;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 13:20 2017/10/13
 */
@Slf4j
@Component
public class PlatformStatusControlHandler extends AbstractHandler {


    @Autowired
    private PlatformMenuService menuService;

    @Override
    public RespEntity handleRequest(SearchRequestAfterOta searchRequestAfterOta) throws SQLException {
        log.info("---------状态控制开始--------- ");
        String cid = searchRequestAfterOta.getCid();

        PlatformMenuVo vo = getVo(cid);

        if (vo == null) {
            return RespEntity.error(RespCode.PLATFORM_CONFIG_EROOOR);
        }
        //判断 此 平台是否上线  如果下线 直接返回
        if (vo.getStatus().equals(0)) {
            return RespEntity.error(RespCode.PLATFORM_STATUS_EROOOR);
        }
        //如果在线 就传给下一个处理对象 进行处理
        if (getNextHandler() != null) {
            log.info("---------状态控制完毕--------- ");
            return nextHandler.handleRequest(searchRequestAfterOta);
        }
        return RespEntity.success(null);
    }

    private PlatformMenuVo getVo(String cid) throws SQLException {
        PlatformMenuVo platformMenuInDb = getPlatformId(cid);

        if (platformMenuInDb == null) {
            log.error("服务平台配置数据库错误 {} ", cid);
            return null;
        }

        Integer plafromId = platformMenuInDb.getId();

        Object statusConfigVo = preFixToBean(RedisPrefix.PLATFORMMENU.getPrefix() + plafromId, redisTemplate, PlatformMenuVo.class);

        PlatformMenuVo menuVo;


        if (statusConfigVo != null) {

            menuVo = (PlatformMenuVo) statusConfigVo;

        } else {
            List<PlatformMenuVo> platformMenuVos = menuService.listPlatformMenuVo(PlatformMenuVo.builder().id(plafromId).build());
            if (platformMenuVos == null || platformMenuVos.size() != 1) {
                log.error("服务平台上下线配置数据库错误 {} ", cid);
                return null;
            }
            menuVo = platformMenuVos.get(0);
            //刷新缓存
            setBeanIntoRedis(menuVo, RedisPrefix.PLATFORMMENU.getPrefix() + cid, redisTemplate, RedisPrefix.PLATFORMMENU);
        }
        return menuVo;
    }

}
