package com.bjlt.spider.chains.searchchaines;

import com.bjlt.spider.chains.abstractchains.AbstractHandler;
import com.bjlt.spider.entity.PlatformMenuVo;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.entity.TimeControConfigVo;
import com.bjlt.spider.search.service.TimeControConfigService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespCode;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.utils.DateCompareUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.SQLException;
import java.util.Date;
import java.util.List;

import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.preFixToBean;
import static com.bjlt.spider.utils.RedisKeyAndBeanUtil.setBeanIntoRedis;

/**
 * @author: Scott
 * @Description: 时间控制处理器
 * @Date: Create in 10:16 2017/10/13
 */
@Slf4j
@Component
public class TimeControHandler extends AbstractHandler {

    @Autowired
    private TimeControConfigService configService;

    @Override
    public RespEntity handleRequest(SearchRequestAfterOta searchRequestAfterOta) throws SQLException {
        log.info("---------时间控制开始--------- ");
        String cid = searchRequestAfterOta.getCid();

        TimeControConfigVo vo = getVo(cid);

        if (vo == null) {
            return RespEntity.error(RespCode.PLATFORM_CONFIG_EROOOR);
        }
        //如果定义的上线时间比今天大 则说明现在还没上线
        if (vo.getSaleDateStart().compareTo(new Date()) >= 1) {
            return RespEntity.error(RespCode.PLATFORM_ONLINE_EROOOR);
        }
        //说明已下线
        if (vo.getSaleDateEnd().compareTo(new Date()) < 1) {
            return RespEntity.error(RespCode.PLATFORM_ONLINE_EROOOR);
        }

        Integer startStatus = DateCompareUtil.compareTwoDays(vo.getDayTimeStart(), new Date());
        if (!startStatus.equals(-1)) {
            return RespEntity.error(RespCode.PLATFORM_ONLINE_EROOOR);
        }
        Integer endStatus = DateCompareUtil.compareTwoDays(vo.getDayTimeEnd(), new Date());

        if (endStatus.equals(-1)) {
            return RespEntity.error(RespCode.PLATFORM_ONLINE_EROOOR);
        }

        if (getNextHandler() != null) {
            log.info("---------时间控制完毕--------- ");
            return nextHandler.handleRequest(searchRequestAfterOta);
        }

        return RespEntity.success(null);


        //return null;
    }

    private TimeControConfigVo getVo(String cid) throws SQLException {
        PlatformMenuVo platformMenuInDb = getPlatformId(cid);

        if (platformMenuInDb == null) {
            log.error("服务平台配置数据库错误 {} ", cid);
            return null;
        }

        Integer plafromId = platformMenuInDb.getId();

        Object timeControConfigObj = preFixToBean(RedisPrefix.DBTIMECONTRO.getPrefix() + plafromId, redisTemplate, TimeControConfigVo.class);

        TimeControConfigVo vo;


        if (timeControConfigObj != null) {

            vo = (TimeControConfigVo) timeControConfigObj;

        } else {
            List<TimeControConfigVo> timeControConfigVos = configService.listTimeControConfig(TimeControConfigVo.builder().platFormId(plafromId).build());
            if (timeControConfigVos == null || timeControConfigVos.size() != 1) {
                log.error("服务平台时间线配置数据库错误TimeControConfigVo  {} ", cid);
                return null;
            }
            vo = timeControConfigVos.get(0);
            //刷新缓存
            setBeanIntoRedis(vo, RedisPrefix.DBTIMECONTRO.getPrefix() + cid, redisTemplate, RedisPrefix.DBTIMECONTRO);
        }
        return vo;
    }

}
