package com.bjlt.spider.search.service.spiderservice.spiderserviceimpl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.config.HandlerInitListener;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.search.service.AirSourceMenuService;
import com.bjlt.spider.search.service.RefundRulesService;
import com.bjlt.spider.search.service.spiderservice.CreateSpiderService;
import com.bjlt.spider.util.MD5Util;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.utils.ActionIdUtils;
import com.bjlt.spider.utils.DateUtils;
import com.bjlt.spider.utils.OtConfigMd5;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 20:23 2017/10/18
 */
@Slf4j
@Service
public class CreateSpiderServiceImpl implements CreateSpiderService, Observer {
    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    @Autowired
    private AirSourceMenuService menuService;

    @Autowired
    private RefundRulesService rulesService;

    @Autowired
    private SpiderObservable spiderObservable;

    private List<RespEntity> entityList;

    public List<RespEntity> getEntityList() {
        return entityList;
    }

    public void setEntityList(List<RespEntity> entityList) {
        this.entityList = entityList;
    }


    public SearchRequestAfterOta getOta() {
        return ota;
    }

    public void setOta(SearchRequestAfterOta ota) {
        this.ota = ota;
    }

    public SearchRequestAfterOta ota;


    @Override
    public RespEntity createSpider(SearchRequestAfterOta otaConfig) throws SQLException {


        this.setOta(otaConfig);

        SearchRequestAfterOta newOtaConfig = null;
        try {
            newOtaConfig = otaConfig.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        newOtaConfig.setCid(null);
        List<AirSourceLineVo> airSourceLines = otaConfig.getAirSourceLines();

        List<AirSourceMenuVo> airSourceMenuVos = Lists.newArrayList();

        Iterator<AirSourceLineVo> iterator = airSourceLines.iterator();


        List<AirSourceLineVo> newAirSourceLines = Lists.newArrayList();

        while (iterator.hasNext()) {

            Object next = iterator.next();
            if (next instanceof AirSourceLineVo) {

                airSourceMenuVos.addAll(menuService.listAirsourceMenu(AirSourceMenuVo.builder().groupId(((AirSourceLineVo) next).getAirSourceID()).build()));

                newAirSourceLines.add((AirSourceLineVo) next);
            } else {

                AirSourceLineVo vo = JSONObject.parseObject(next.toString(), AirSourceLineVo.class);

                airSourceMenuVos.addAll(menuService.listAirsourceMenu(AirSourceMenuVo.builder().groupId(vo.getAirSourceID()).build()));

                newAirSourceLines.add(vo);
            }

        }

       Set<AirSourceMenuVo> airSourceMenuVosSet = new HashSet<>(airSourceMenuVos);

        //数据包装
        List<SearchRequestEntity> requestEntities = Lists.newArrayList();

        newAirSourceLines.forEach(s ->
                airSourceMenuVosSet.forEach(t -> {
                    if (s.getAirSourceID().equals(t.getGroupId())) {
                        SearchRequestEntity entity = new SearchRequestEntity();
                        entity.setAirSource(String.valueOf(s.getAirSourceID()));
                        entity.setToCity(s.getToCity());
                        entity.setFormCity(s.getFromCity());
                        entity.setFromDate(DateUtils.changeDateFormat(otaConfig.getFromDate()));
                        if (!StringUtils.isBlank(otaConfig.getRetDate())) {
                            entity.setRetDate(DateUtils.changeDateFormat(otaConfig.getRetDate()));
                        }
                        entity.setAdultNum(s.getAdultNum());
                        entity.setSearchUrl(t.getSearchUrl());
                        entity.setTripType(otaConfig.getTripType());
                        entity.setIsDefault(t.getIsDefault());
                        requestEntities.add(entity);
                    }
                })
        );


        Set<SearchRequestEntity> requestEntitiesSet = new HashSet(requestEntities);

        //根据航司 ID 把所有的请求地址拿出来
        Map<String, Object> retMap = reduceSpider(requestEntitiesSet);

        List<SearchResponseEntity> searchResponseEntityList = Lists.newArrayList();

        List<RespEntity> respEntities = Lists.newArrayList();
//


        // String retEntity = redisTemplate.opsForValue().get("RET-ENTITY" + newOtaConfig.toString());
        String key = OtConfigMd5.toMd5(newOtaConfig);

        //这是新 搜到的
        Set<String> keys = redisTemplate.keys("RET-NEW-" + key + "*");

        List<RespEntity> finalRespEntities = Lists.newArrayList();
        keys.parallelStream().forEach(t -> {
            String s = redisTemplate.opsForValue().get(t);
            RespEntity RespEntity = (RespEntity) JSONObject.parse(s);
            finalRespEntities.add(RespEntity);
        });
        //这是以前的
        log.info("开始搜索缓存中的结果的结果");
        Set<String> keysOld = redisTemplate.keys("RET-OLD-" + key + "-" + "*");
        keysOld.parallelStream().forEach(t -> {
            String s = redisTemplate.opsForValue().get(t);
            finalRespEntities.add(JSONObject.parseObject(s, RespEntity.class));
        });

        if (keysOld.size() == 0) {
            log.info("缓存中没有，{}---->{}", otaConfig.getCid(), otaConfig.getFromCity(), otaConfig.getToCity());
            spiderObservable.distributeSpider(retMap);
            respEntities = getEntityList();
        }
        if (respEntities ==  null){
            log.error("获取数据失败"+otaConfig.toString());
            return new RespEntity(2333, "获取数据失败");
        }

        finalRespEntities.addAll(respEntities);
        List<RespEntity> collect = Lists.newArrayList();
        if (finalRespEntities.size() != 0){
            collect = finalRespEntities.stream().filter(t -> t.getStatus() == 200).collect(Collectors.toList());

        }


        if (collect == null || collect.size() == 0) {
            log.error("获取数据失败"+otaConfig.toString());
            return new RespEntity(2333, "获取数据失败");
        }
        //刷新数据
        List<String> finalJsonList = Lists.newArrayList();
        collect.parallelStream().forEach(t -> {
                    String jsEntity = JSONObject.toJSONString(t);
                    finalJsonList.add(jsEntity);

                }
        );

        Set<String> set = new HashSet<>(finalJsonList);

        //刷新
        keysOld.parallelStream().forEach(t -> {
            redisTemplate.delete(t);
        });
        log.info("将数据存入缓存{}", otaConfig.toString());
        set.parallelStream().forEach(t ->

                redisTemplate.opsForValue().set("RET-OLD-" + key + "-" + ActionIdUtils.getUUID(), t, 30, TimeUnit.MINUTES)


        );

        log.info("---------开始处理结果---------");
        for (Object o : set) {


            log.info("当前结果是 {}", o);
            ObjectMapper objectMapper = new ObjectMapper();
            RespEntity entity = null;
            try {
                entity = objectMapper.readValue(o.toString(), RespEntity.class);
            } catch (IOException e) {
                e.printStackTrace();
            }

            if (entity != null && entity.getStatus() == 200) {


                Object ret = entity.getAttachment();


                String s = JSONObject.toJSONString(ret);


                SearchResponseEntity responseEntity = null;
                try {
                    responseEntity = objectMapper.readValue(s, SearchResponseEntity.class);


                    searchResponseEntityList.add(responseEntity);

                    log.info("开始做笛卡尔积");
                    searchResponseEntityList = descartesLines(searchResponseEntityList);


                } catch (IOException e) {
                    e.printStackTrace();
                }

            }
        }

        return RespEntity.success(searchResponseEntityList);
    }
    //returnSearchInfo();


    @Override
    public Map<String, Object> reduceSpider(Set<SearchRequestEntity> requestEntities) {

        //根据航司分组

        Map<String, List<SearchRequestEntity>> collectBySource = requestEntities.stream().collect(Collectors.groupingBy(t -> t.getAirSource()));

        List<SearchRequestEntity> defaultEntity = Lists.newArrayList();


        List<Object> otherEntity = Lists.newArrayList();


        Map<String, Object> retMap = Maps.newHashMap();

        //先把默认的数据源取出来
//
        collectBySource.forEach((k, v) ->

                defaultEntity.add(v.stream().filter(s -> s.getIsDefault().equals(1)).collect(Collectors.toList()).get(0))
        );


        collectBySource.forEach((k, v) ->
                otherEntity.add(v.stream().filter(s -> !s.getIsDefault().equals(1)).collect(Collectors.toList()))

        );

        retMap.put("default", defaultEntity);
        retMap.put("unDefault", otherEntity);
        return retMap;

    }


    public List<SearchResponseEntity> returnSearchInfo(List<RespEntity> respEntityList) {

        List<SearchResponseEntity> searchResponseEntityList = Lists.newArrayList();

        for (RespEntity entity : respEntityList) {
            if (entity.getStatus() == 200) {


                List list = (List) entity.getAttachment();

                list.forEach(t -> {

                            String s = JSONObject.toJSONString(t);

                            ObjectMapper objectMapper = new ObjectMapper();


                            SearchResponseEntity responseEntity = null;
                            try {
                                responseEntity = objectMapper.readValue(s, SearchResponseEntity.class);

                                String actionId = "data" + ActionIdUtils.getUUID();

                                responseEntity.setData(actionId);
                                //TODO 此处要修改


                                try {
                                    insertRefundInfo(responseEntity);
                                } catch (Exception e) {
                                    log.error("匹配退改签失败");
                                }

                                searchResponseEntityList.add(responseEntity);
                            } catch (IOException e) {
                                e.printStackTrace();
                            }

                        }
                );
            }


        }

        return searchResponseEntityList;


    }


//        List<RespEntity> ret = Lists.newArrayList();
//        defaultList.parallelStream().forEach(t -> {
//
//
//            RespEntity respEntity;
//            String s = redisTemplate.opsForValue().get(t);
//
//            //首先在缓存中那
//
//            if (s != null) {
//                try {
//                    respEntity = JSONObject.parseObject(s, RespEntity.class);
//
//                    ret.add(respEntity);
//                } catch (Exception e) {
//                    e.printStackTrace();
//                }
//                return;
//            }


    //    final CompletableFuture<RespEntity> future = CompletableFuture.supplyAsync(() -> getDefaultRet(t), executorService);


    // try {
    //这个时间是默认源的等待执行时间
//                respEntity = future.get(15, TimeUnit.SECONDS);
//
//                if (respEntity != null && respEntity.getStatus() != 200) {
//                    log.error("默认源执行失败，切换至其他源 {}", t.toString());
//                    respEntity = doNotDefault(t);
//                }
//
//                ret.add(respEntity);

//            } catch (Exception e) {
//                e.printStackTrace();
//                log.error("默认源执行异常，切换至其他源 {}", t.toString());
////                respEntity = doNotDefault(t);
////                ret.add(respEntity);
//                //executorService.shutdownNow();
//            }


//            try {
////                respEntity = future.get(30, TimeUnit.SECONDS);
////                if (respEntity != null && respEntity.getStatus() != 200) {
////                    //如果30秒拿到结果 则 扔进缓存中
////                    redisTemplate.opsForValue().set(t.toString(), respEntity.toString(),1,TimeUnit.HOURS);
////                }
//            } catch (InterruptedException e) {
//                e.printStackTrace();
//            } catch (ExecutionException e) {
//                e.printStackTrace();
//            } catch (TimeoutException e) {
//                e.printStackTrace();
//            }

//
//    public RespEntity getDefaultRet(SearchRequestEntity entity) {
//
//        RespEntity respEntity = null;
//        try {
//            respEntity = HandlerInitListener.searchMethodMap.get(entity.getAirSource()).webSearch(entity);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//
//
//        return respEntity;
//    }

    /**
     * 暂时不用了 随机切换其他源
     *
     * @param entity
     * @return
     */
    public RespEntity getNotDefaultRet(SearchRequestEntity entity) {

        String airSource = entity.getAirSource();
        List<AirSourceMenuVo> airSourceMenuVos = Lists.newArrayList();
        try {
            airSourceMenuVos = menuService.listAirsourceMenu(AirSourceMenuVo.builder().groupId(airSource).build());
        } catch (SQLException e) {
            e.printStackTrace();
        }

        //随机去一个
        String postfix = airSourceMenuVos.get(new Random().nextInt(airSourceMenuVos.size())).getPostfix();
        //暂时只能这么写 TODO
        if ("APP".equals(postfix)) {
            log.error("正在切换备用源  {}", entity.toString());
            return HandlerInitListener.searchMethodMap.get(entity.getAirSource()).webSearch(entity);
        } else {
            log.error("正在切换备用源  {}", entity.toString());
            return HandlerInitListener.searchMethodMap.get(entity.getAirSource()).webSearch(entity);
        }
    }

//
//    public RespEntity doNotDefault(SearchRequestEntity entity) {
//
//
//        final CompletableFuture<RespEntity> future =
//                CompletableFuture.supplyAsync(() -> getNotDefaultRet(entity), executorService);
//        RespEntity respEntity = null;
//        try {
//            respEntity = future.get(10, TimeUnit.SECONDS);
//
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return respEntity;
//    }


    /**
     * 匹配退改签
     */
    public void insertRefundInfo(SearchResponseEntity searchResponseEntity) {


        //根据 SOURCENAME 找退改签匹配

        String airSourceName = searchResponseEntity.getFromInformation().getAirSourceName();

        Integer sourceId = SourceEnum.valueOf(airSourceName).getSourceId();
        List<RefundRules> refundRules = rulesService.listRefundRules(RefundRules.builder().sourceGroupId(sourceId).build());

        if (refundRules == null || refundRules.size() != 1) {
            return;
        }
        RefundRules refundRulesUse = refundRules.get(0);

        RuleEntity ruleEntity = RuleEntity.builder()
                .baggage(refundRulesUse.getBaggage())
                .endorse(refundRulesUse.getEndorse())
                .endorsement(refundRulesUse.getEndorsement())
                .hasBaggage(refundRulesUse.getHasBaggage())
                .hasEndorse(refundRulesUse.getHasEndorse())
                .hasNoShow(refundRulesUse.getHasNoShow())
                .noShowLimitTime(refundRulesUse.getNoShowLimitTime())
                .partEndorse(refundRulesUse.getPartEndorse())
                .specialNoShow(refundRulesUse.getSpecialNoShow())
                .hasRefund(refundRulesUse.getHasRefund())
                .other(refundRulesUse.getOther())
                .refund(refundRulesUse.getRefund())
                .partRefundPrice(refundRulesUse.getPartEndorsePrice())
                .penalty(refundRulesUse.getPenalty())
                .partEndorsePrice(refundRulesUse.getPartEndorsePrice())
                .partRefund(refundRulesUse.getPartRefund())
                .build();

        searchResponseEntity.setRuleEntity(ruleEntity);
    }

    /**
     * 笛卡尔积
     *
     * @param List
     */
    public List<SearchResponseEntity> descartesLines(List<SearchResponseEntity> List) {
        //迭代 将所有的去程拿出来
        List<AirlineInfomation> fromList = Lists.newArrayList();
        //将所有的返程拿出来
        List<AirlineInfomation> retList = Lists.newArrayList();

        List<SearchResponseEntity> finalList = Lists.newArrayList();

        List.parallelStream().forEach(t -> {
                    fromList.addAll(t.getFromSegments());
                    retList.addAll(t.getRetSegments());
                }

        );

        SearchResponseEntity oldEntity = List.get(0);


        if (retList.size() == 0) {
            fromList.stream().forEach(f -> {
                        SearchResponseEntity searchResponseEntity = new SearchResponseEntity();
                        String data = "data" + ActionIdUtils.getUUID();
                        searchResponseEntity.setData(data);
                        searchResponseEntity.setFromDate(oldEntity.getFromDate());
                        searchResponseEntity.setRetDate(oldEntity.getRetDate());
                        searchResponseEntity.setFormCity(oldEntity.getFormCity());
                        searchResponseEntity.setToCity(oldEntity.getToCity());
                        searchResponseEntity.setCurreny(f.getCurrency());
                        searchResponseEntity.setFromInformation(f);
                        searchResponseEntity.setAdultPrice(f.getAdultPrice());
                        searchResponseEntity.setAdultPriceTax(f.getAdultTax());
                        searchResponseEntity.setChildPrice(f.getAdultPrice());
                        searchResponseEntity.setChildPriceTax(f.getAdultTax());
                        searchResponseEntity.setDurationTotal(f.getDurationTotal());
                        insertRefundInfo(searchResponseEntity);
                        finalList.add(searchResponseEntity);
                        redisTemplate.opsForValue().set(data, JSONObject.toJSONString(searchResponseEntity), 1, TimeUnit.DAYS);


                    }


            );

        } else {
            fromList.stream().forEach(f ->

                    retList.stream().forEach(r -> {
                        SearchResponseEntity searchResponseEntity = new SearchResponseEntity();
                        String data = "data" + ActionIdUtils.getUUID();
                        searchResponseEntity.setData(data);
                        searchResponseEntity.setFromDate(oldEntity.getFromDate());
                        searchResponseEntity.setRetDate(oldEntity.getRetDate());
                        searchResponseEntity.setFormCity(oldEntity.getFormCity());
                        searchResponseEntity.setToCity(oldEntity.getToCity());
                        searchResponseEntity.setCurreny(f.getCurrency());
                        searchResponseEntity.setFromInformation(f);
                        searchResponseEntity.setRetInformation(r);
                        searchResponseEntity.setAdultPrice(f.getAdultPrice().add(r.getAdultPrice()));
                        searchResponseEntity.setAdultPriceTax(f.getAdultTax().add(r.getAdultTax()));
                        searchResponseEntity.setChildPrice(f.getAdultPrice().add(r.getAdultPrice()));
                        searchResponseEntity.setChildPriceTax(f.getAdultTax().add(r.getAdultTax()));
                        searchResponseEntity.setDurationTotal(f.getDurationTotal() + r.getDurationTotal());
                        insertRefundInfo(searchResponseEntity);
                        finalList.add(searchResponseEntity);
                        redisTemplate.opsForValue().set(data, JSONObject.toJSONString(searchResponseEntity), 1, TimeUnit.DAYS);
                    })
            );
        }
        return finalList;


    }

    @Override
    public void update(Observable o, Object arg) {
        log.info("监听器收到通知{}", ota.toString());
        SpiderObservable spiderObservable = (SpiderObservable) o;
        setEntityList(spiderObservable.getEntityList());
    }
}

