package com.bjlt.spider.search.service.spiderservice.spiderserviceimpl;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.entity.SearchRequestEntity;
import com.bjlt.spider.spiderhandler.DefaultSpiderHandler;
import com.bjlt.spider.util.MD5Util;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.utils.ActionIdUtils;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 11:13 2017/10/27
 */
@Slf4j
@Component
public class SpiderObservable extends Observable {


    @Autowired
    private CreateSpiderServiceImpl createSpiderService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    static ThreadFactory factory = new ThreadFactoryBuilder()
            .setNameFormat("search-pool-%d").build();

    private static ExecutorService executorService =
            new ThreadPoolExecutor(30, 400, 30L, TimeUnit.SECONDS,
                    new LinkedBlockingDeque<>(1024), factory, new ThreadPoolExecutor.DiscardPolicy());

    private List<RespEntity> entityList;


    public List<RespEntity> getEntityList() {
        return entityList;
    }

    public void setEntityList(List<RespEntity> entityList) {
        this.entityList = entityList;
        setChanged();
        notifyObservers();
    }


    public void distributeSpider(Map<String, Object> retMap) {
        AtomicInteger atomicInteger = new AtomicInteger(0);


        //首先分发默认爬虫
        List<SearchRequestEntity> defaultList = (List<SearchRequestEntity>) retMap.get("default");


        //装配线程引擎


        log.info("---------装配爬虫引擎开始---------");

        log.info("---------装配爬虫引擎完毕---------");

        List<RespEntity> ret = Lists.newArrayList();

        defaultList.parallelStream().forEach(f -> {

            final CompletableFuture<RespEntity> future =
                    CompletableFuture.supplyAsync(() -> new DefaultSpiderHandler(f).getDefult(), executorService);

            RespEntity respEntity = null;

            //最多等待10S 10S 之内返回多少是多少
            Long start = System.currentTimeMillis();
            log.info("-----{} 航司开始执行时间是 {}------", f.getAirSource(), start);
            try {
                respEntity = future.get(15, TimeUnit.SECONDS);
                long now = System.currentTimeMillis();
                log.info("在十秒前秒之前拿到 {} 数据,--- 共用时{} 秒", f.getAirSource(), (now - start) / 1000);
                if (respEntity.getStatus() == 200){
                    ret.add(respEntity);
                }

                if (ret.size() == defaultList.size()) {
                    if (atomicInteger.incrementAndGet() == 1) {
                        now = System.currentTimeMillis();
                        log.info("在十秒之前拿到所有航司的数据,--- 共用时{} 秒", (now - start) / 1000);
                        setEntityList(ret);
                        setChanged();
                        notifyObservers();
                    }
                }

            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (ExecutionException e) {
                e.printStackTrace();
            } catch (TimeoutException e) {


                if (atomicInteger.incrementAndGet() == 1) {
                    setEntityList(ret);
                    setChanged();
                    notifyObservers();
                }

                try {
                    respEntity = future.get(20, TimeUnit.SECONDS);


                    if (respEntity.getStatus() == 200) {

                        String jsEntity = JSONObject.toJSONString(respEntity);

                        log.info("后续拿到了数据{}",createSpiderService.getOta().toString());
                        String key = MD5Util.encrypt(createSpiderService.getOta().toString());


                        redisTemplate.opsForValue().set("RET-NEW-" + key + "-" + ActionIdUtils.getUUID(), jsEntity, 30, TimeUnit.MINUTES);

                    }


                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                } catch (ExecutionException e1) {
                    e1.printStackTrace();
                } catch (TimeoutException e1) {
                    e1.printStackTrace();
                }
            }
            Long now = System.currentTimeMillis();
            log.info("-----{} 航司结束执行时间是 {}------", f.getAirSource(), (now - start) / 1000);


        });

    }

}
