package com.bjlt.spider.utils;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.util.RedisPrefix;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 11:31 2017/10/13
 */
@Slf4j
public class RedisKeyAndBeanUtil {


    /**
     *  通过 key 取出JSON 并转换成对象
     * @param key
     * @param redisTemplate
     * @param clazz
     * @return
     */
    public static Object preFixToBean(String key, RedisTemplate<String, String> redisTemplate, Class clazz) {
        //通过缓存前缀取出对象
        String ret = redisTemplate.opsForValue().get(key);
        //如果缓存中取不到直接返回 null
        if (StringUtils.isBlank(ret)){
            return null;
        }
        JSONObject jsonObject = null;
        Object o = null;
        try {
            jsonObject = JSONObject.parseObject(ret);
            o = jsonObject.toJavaObject(clazz);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("JSON {} 转 OBJ 失败", jsonObject);
        }
        return o;

    }


    /**
     *  通过 key 取出JSON 并转换成对象
     * @param key
     * @param redisTemplate
     * @param clazz
     * @return
     */
    public static Set<String> preFixToSet(String key, RedisTemplate<String, String> redisTemplate, Class clazz) {
        //通过缓存前缀取出对象
        Set<String> set = redisTemplate.opsForSet().members(key);
        //如果缓存中取不到直接返回 null
        if (set == null || set.size() == 0){
            return null;
        }
        Set ret = Sets.newHashSet();

        try {
            set.forEach(t->

               ret.add(JSONObject.parseObject(t).toJavaObject(clazz))

            );
        } catch (Exception e) {
            e.printStackTrace();
            log.error("key {} 取值 失败", key);
        }
        return set;

    }

    /**
     * 将对象解析为 JSON 并存入 redis 中
     */
    public static void setBeanIntoRedis(Object source,String key,RedisTemplate<String, String> redisTemplate,RedisPrefix redisPrefix){

        String target = JSONObject.toJSONString(source);

        redisTemplate.opsForValue().set(key,target, redisPrefix.getExpriedTime(), TimeUnit.HOURS);

    }


    /**
     * 将对象解析为 JSON 并存入 redis 中  结构为 set
     */
    public static void setBeanIntoRedisWithSet(Object source,String key,RedisTemplate<String, String> redisTemplate){

        String target = JSONObject.toJSONString(source);

        redisTemplate.opsForSet().add(key,target);

    }
}
