package com.bjlt.spider.verify.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.interfaces.ControRpc;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.utils.DateUtils;
import com.bjlt.spider.verify.service.VerifyService;
import com.bjlt.spider.verify.utils.AirLineInformationCompare;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:10 2017/10/23
 */
@Slf4j
@Service
public class VerifyServiceImpl implements VerifyService {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private ControRpc controRpc;


    @Override
    public RespEntity verify(VerifyRequestAfterOta ota) {

        //取出 ota 里面的 航班信息
        String data = ota.getData();

        String entityString = redisTemplate.opsForValue().get(data);

        SearchResponseEntity entity = JSONObject.parseObject(entityString, SearchResponseEntity.class);

        //  List<RouteInfoEntity> fromSegmentsFromRedis = entity.getFromSegments();

        //  List<RouteInfoEntity> retSegmentsFromRedis = entity.getRetSegments();


        SearchRequestAfterOta searchRequestAfterOta = new SearchRequestAfterOta();

        if (ota.getChildNum() != null) {
            searchRequestAfterOta.setChildNum(Integer.valueOf(ota.getChildNum()));
        }


        searchRequestAfterOta.setAdultNum(Integer.valueOf(ota.getAdultNum()));

        searchRequestAfterOta.setCid(ota.getCid());

        searchRequestAfterOta.setFromCity(entity.getFormCity());

        searchRequestAfterOta.setToCity(entity.getToCity());

        searchRequestAfterOta.setFromDate(DateUtils.changeDateFormatforVerify(entity.getFromDate()));

        searchRequestAfterOta.setRetDate(DateUtils.changeDateFormatforVerify(entity.getRetDate()));

        searchRequestAfterOta.setTripType(ota.getTripType());

        RespEntity spiderEntity = null;
        //现在去搜索
        try {
            spiderEntity = controRpc.createSpider(searchRequestAfterOta);
        } catch (SQLException e) {
            e.printStackTrace();
        }


        Map<AirlineInfomation, AirlineInfomation> compareInformationMap = Maps.newHashMap();


        Map<String, BigDecimal> priceMap = Maps.newHashMap();


        //这是我搜出来的
        if (spiderEntity.getStatus() == 200) {
            Object o = spiderEntity.getAttachment();

            List list = (List) o;


            list.forEach(t -> {

                        String searchEntityString = JSONObject.toJSONString(t);

                        ObjectMapper objectMapper = new ObjectMapper();

                        SearchResponseEntity responseEntityFromVerify = null;


                        try {
                            responseEntityFromVerify = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);

                            priceMap.put(responseEntityFromVerify.getData(), responseEntityFromVerify.getAdultPrice());


                            AirlineInfomation fromAirlineInfomationFromRedis = entity.getFromInformation();

                            AirlineInfomation retAirlineInfomationFromRedis = entity.getRetInformation();

                            AirlineInfomation fromAirlineInfomationFromSearch = responseEntityFromVerify.getFromInformation();

                            AirlineInfomation retAirlineInfomationFromSearch = responseEntityFromVerify.getRetInformation();


                            //只验证单程
                            if (SearchRequestEntity.ONE_WAY.equals(ota.getTripType())) {

                                compareInformationMap.putAll(AirLineInformationCompare.compareInformation(fromAirlineInfomationFromRedis, fromAirlineInfomationFromSearch));
                            }
                            //需要验证单程和返程
                            if (SearchRequestEntity.DOU_WAY.equals(ota.getTripType())) {
                                compareInformationMap.putAll(AirLineInformationCompare.compareInformation(fromAirlineInfomationFromRedis, fromAirlineInfomationFromSearch));
                                compareInformationMap.putAll(AirLineInformationCompare.compareInformation(retAirlineInfomationFromRedis, retAirlineInfomationFromSearch));
                            }

                        } catch (IOException e) {
                            e.printStackTrace();
                            System.out.println(t);
                        }

                    }

            );

        }


        VerifyResponseEntity verifyResponseEntity = new VerifyResponseEntity();

        //在 OTAAPI 里面 进行价格分析  此时不做价格校验 只判断航班还在不在
        if (SearchRequestEntity.ONE_WAY.equals(ota.getTripType())) {

            if (compareInformationMap.size() == 0) {
                return new RespEntity(222, "航班验证失败");
            }

            verifyResponseEntity.setFromSegments(entity.getFromInformation().getRouteInfoEntities());
            verifyResponseEntity.setData(data);
            FiledInsert(verifyResponseEntity, entity, compareInformationMap);


        }
        if (SearchRequestEntity.DOU_WAY.equals(ota.getTripType())) {
            //如果是往返 就必须判断来回信息
            if (compareInformationMap.size() == 0) {
                return new RespEntity(222, "航班验证失败");
            }


            verifyResponseEntity.setFromSegments(entity.getFromInformation().getRouteInfoEntities());
            verifyResponseEntity.setData(data);
            verifyResponseEntity.setRetSegments(entity.getRetInformation().getRouteInfoEntities());
            FiledInsert(verifyResponseEntity, entity, compareInformationMap);


        }


        return RespEntity.success(verifyResponseEntity);
    }


    private void FiledInsert(VerifyResponseEntity verifyResponseEntity, SearchResponseEntity
            entity, Map<AirlineInfomation, AirlineInfomation> compareInformationMap) {

        BigDecimal adultPriceInVerify = new BigDecimal(0);
        BigDecimal adultTaxInVerify = new BigDecimal(0);

        Iterator<AirlineInfomation> iterator = compareInformationMap.values().iterator();
        while (iterator.hasNext()) {
            AirlineInfomation next = iterator.next();
            adultPriceInVerify = adultPriceInVerify.add(next.getAdultPrice());
            adultTaxInVerify = adultTaxInVerify.add(next.getAdultTax());

        }


        //这是验证的价格
        verifyResponseEntity.setCurrency(entity.getCurreny());

        verifyResponseEntity.setAdultPriceTotal(adultPriceInVerify);

        verifyResponseEntity.setChildPriceTotal(adultPriceInVerify);


        verifyResponseEntity.setAdultTax(adultTaxInVerify);
        verifyResponseEntity.setChildTax(adultTaxInVerify);

        //这是搜索出来的价格
        verifyResponseEntity.setAdultPriceTotalInData(entity.getAdultPrice());


//        verifyResponseEntity.setAdultTax(entity.getAdultTax());

//
//        verifyResponseEntity.setChildTax(entity.getAdultTax());
//
//        verifyResponseEntity.setAdultTaxType(entity.getAdultTaxType());
//
//        verifyResponseEntity.setChildTaxType(entity.getAdultTaxType());

//        verifyResponseEntity.setTicketTimeLimit(entity.getTicketTimeLimit());
//
//        verifyResponseEntity.setTicketInvoiceType(entity.getTicketInvoiceType());
//
//        verifyResponseEntity.setNationality(entity.getNationality());
//
//        verifyResponseEntity.setNationalityType(entity.getNationalityType());
//
//        verifyResponseEntity.setSuitAge(entity.getSuitAge());
//
//
//        verifyResponseEntity.setPriceType(entity.getPriceType());
//
//        verifyResponseEntity.setApplyType(entity.getApplyType());

    }


}
